﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Inc.
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Configuration;

namespace EaseFilter.CommonObjects
{
    public class RegistryFilterRuleSection : ConfigurationSection
    {
        [ConfigurationProperty("", IsRequired = true, IsDefaultCollection = true)]
        public RegistryFilterRuleCollection Instances
        {
            get { return (RegistryFilterRuleCollection)this[""]; }
            set { this[""] = value; }
        }
    }

    public class RegistryFilterRuleCollection : ConfigurationElementCollection
    {
        public RegistryFilterRule this[int index]
        {
            get { return (RegistryFilterRule)BaseGet(index); }
            set
            {
                if (BaseGet(index) != null)
                {
                    BaseRemoveAt(index);
                }
                BaseAdd(index, value);
            }
        }

        public void Add(RegistryFilterRule RegistryFilterRule)
        {
            BaseAdd(RegistryFilterRule);
        }

        public void Clear()
        {
            BaseClear();
        }

        public void Remove(RegistryFilterRule RegistryFilterRule)
        {
            BaseRemove(RegistryFilterRule.ProcessId.ToString() + RegistryFilterRule.ProcessNameFilterMask);
        }

        public void RemoveAt(int index)
        {
            BaseRemoveAt(index);
        }

        public void Remove(string name)
        {
            BaseRemove(name);
        }

        protected override ConfigurationElement CreateNewElement()
        {
            return new RegistryFilterRule();
        }

        protected override object GetElementKey(ConfigurationElement element)
        {
            //The key is the process name + user name
            return ((RegistryFilterRule)element).ProcessId.ToString() + ((RegistryFilterRule)element).ProcessNameFilterMask;
        }
    }

    public class RegistryFilterRule : ConfigurationElement
    {
        /// <summary>
        /// Control the registry access for the process with this process Id. 
        /// </summary>
        [ConfigurationProperty("processId", IsRequired = false)]
        public string ProcessId
        {
            get { return (string)base["processId"]; }
            set { base["processId"] = value; }
        }

        /// <summary>
        /// Control the registry access for the process with this process name if the process Id is 0, or it will skip it. 
        /// </summary>
        [ConfigurationProperty("processNameFilterMask", IsKey = false, IsRequired = false)]
        public string ProcessNameFilterMask
        {
            get { return (string)base["processNameFilterMask"]; }
            set { base["processNameFilterMask"] = value; }
        }

        /// <summary>
        /// Control the registry access for the process with this user name
        /// </summary>
        [ConfigurationProperty("userName", IsRequired = false)]
        public string UserName
        {
            get { return (string)base["userName"]; }
            set { base["userName"] = value; }
        }


        /// <summary>
        /// The the flag to control how to access the registry for the matched process or user
        /// </summary>
        [ConfigurationProperty("accessFlag", IsRequired = true)]
        public uint AccessFlag
        {
            get { return (uint)base["accessFlag"]; }
            set { base["accessFlag"] = value; }
        }

        /// <summary>
        /// Register the callback when the registry access notification was triggered
        /// </summary>
        [ConfigurationProperty("regCallbackClass", IsRequired = true)]
        public ulong RegCallbackClass
        {
            get { return (ulong)base["regCallbackClass"]; }
            set { base["regCallbackClass"] = value; }
        }


        /// <summary>
        /// If it is true, the registry access from the matched process or user will be excluded.
        /// </summary>
        [ConfigurationProperty("isExcludeFilter", IsRequired = true)]
        public bool IsExcludeFilter
        {
            get { return (bool)base["isExcludeFilter"]; }
            set { base["isExcludeFilter"] = value; }
        }

        public RegistryFilterRule Copy()
        {
            RegistryFilterRule dest = new RegistryFilterRule();
            dest.ProcessId = ProcessId;
            dest.ProcessNameFilterMask = ProcessNameFilterMask;
            dest.UserName = UserName;
            dest.AccessFlag = AccessFlag;
            dest.RegCallbackClass = RegCallbackClass;
            dest.IsExcludeFilter = IsExcludeFilter;

            return dest;
        }

    }


}
