﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseFilter Technologies
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
//    NOTE:  THIS MODULE IS UNSUPPORTED SAMPLE CODE
//
//    This module contains sample code provided for convenience and
//    demonstration purposes only,this software is provided on an 
//    "AS-IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, 
//     either express or implied.  
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using Microsoft.Win32.SafeHandles;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.Threading;
using System.Diagnostics;
using System.Management;
using System.Collections;
using System.Configuration;
using System.Windows.Forms;

using EaseFilter.CommonObjects;

namespace  SecureShareManager
{

    public class DRServer
    {

        public DRServer()
        {
        }


        static private bool GetDRInfo(Byte[] drDataBuffer, uint drBufferLength, ref DRPolicyData drPolicyData, ref string lastError)
        {
            Boolean retVal = false;

            try
            {

                MemoryStream ms = new MemoryStream(drDataBuffer);
                BinaryReader br = new BinaryReader(ms);

                drPolicyData.AESVerificationKey = br.ReadUInt32();

                if (FilterAPI.AES_TAG_KEY != drPolicyData.AESVerificationKey)
                {
                    lastError = "The DRPolicyData buffer was corrupted.";
                    return false;
                }

                drPolicyData.AESFlags = (AESFlags)br.ReadUInt32();
                drPolicyData.IVLength = br.ReadUInt32();
                drPolicyData.IV = br.ReadBytes(16);
             
                retVal = true;

            }
            catch (Exception ex)
            {
                retVal = false;
                lastError = "Get DRPolicyData failed with error " + ex.Message;
            }

            return retVal;
        }

        static private bool IsFileAccessAuthorized(string fileName,
                                            string userName,
                                            string processName,
                                            string encryptionIV,
                                            ref string encryptKey,
                                            ref uint accessFlag,
                                            ref string lastError)
        {
            Boolean retVal = true;
            string expireTime = string.Empty;
            string authorizedProcessNames = string.Empty;
            string unauthorizedProcessNames = string.Empty;
            string authorizedUserNames = string.Empty;
            string unauthorizedUserNames = string.Empty;
            string accessFlags = string.Empty;
            
            lastError = string.Empty;
            DateTime currentTime = DateTime.Now;

            try
            {
                string drFilePath = GlobalConfig.DRInfoFolder + "\\" + encryptionIV + ".xml";
                Dictionary<string, string> keyValues = new Dictionary<string, string>();

                if (!File.Exists(drFilePath))
                {
                    lastError = "The meta data file " + drFilePath + " doesn't exist.";
                    return false;
                }

                Utils.LoadAppSetting(drFilePath, ref keyValues);

                encryptKey = string.Empty;
                keyValues.TryGetValue("key", out encryptKey);
                if (string.IsNullOrEmpty(encryptKey))
                {
                    lastError = "The encryption key is empty.";
                    return false;
                }

                keyValues.TryGetValue("accessFlags", out accessFlags);
                uint.TryParse(accessFlags, out accessFlag);
                keyValues.TryGetValue("expireTime", out expireTime);
                keyValues.TryGetValue("authorizedProcessNames", out authorizedProcessNames);
                keyValues.TryGetValue("unauthorizedProcessNames", out unauthorizedProcessNames);
                keyValues.TryGetValue("authorizedUserNames", out authorizedUserNames);
                keyValues.TryGetValue("unauthorizeUserNames", out unauthorizedUserNames);

                if (!string.IsNullOrEmpty(expireTime))
                {
                    DateTime exTime = DateTime.FromFileTime(long.Parse(expireTime));

                    if (currentTime > exTime)
                    {
                        lastError = "the file is expired,current time:" + currentTime.ToString("yyyy-MM-dd-HH-mm-ss") + ",expire time:" + exTime.ToString("yyyy-MM-dd-HH-mm-ss");
                        return false;
                    }
                }

                if (!string.IsNullOrEmpty(authorizedProcessNames))
                {
                    if (authorizedProcessNames.IndexOf(processName.ToLower().Trim()) < 0)
                    {
                        lastError = "the process " + processName + " is not in authorized process list.";
                        return false;
                    }
                }

                if (!string.IsNullOrEmpty(unauthorizedProcessNames))
                {
                    if (unauthorizedProcessNames.IndexOf(processName.ToLower().Trim()) >= 0)
                    {
                        lastError = "the process " + processName + " is in unauthorized process list.";
                        return false;
                    }
                }

                if (!string.IsNullOrEmpty(authorizedUserNames))
                {
                    if (authorizedUserNames.IndexOf(userName.ToLower().Trim()) < 0)
                    {
                        lastError = "the user " + userName + " is not in authorized user list.";
                        return false;
                    }
                }

                if (!string.IsNullOrEmpty(unauthorizedUserNames))
                {
                    if (unauthorizedUserNames.IndexOf(userName.ToLower().Trim()) >= 0)
                    {
                        lastError = "the user " + userName + " is in unauthorized user list.";
                        return false;
                    }
                }


            }
            catch (Exception ex)
            {
                EventManager.WriteMessage(286, "IsFileAccessAuthorized", EventLevel.Error, "Get file " + fileName + "permission failed with exception:" + ex.Message);
                retVal = false;
            }

            return retVal;

        }

        static public bool GetFileAccessPermission(ref FilterAPI.MessageSendData messageSend, ref FilterAPI.MessageReplyData messageReply)
        {
            Boolean retVal = true;
            string fileName = messageSend.FileName;
            string lastError = string.Empty;
            string processName = string.Empty;
            string userName = string.Empty;
            string encryptKey = string.Empty;


            try
            {

                FilterAPI.DecodeProcessName(messageSend.ProcessId, out processName);
                FilterAPI.DecodeUserName(messageSend.Sid, out userName);

                DRPolicyData drPolicyData = new DRPolicyData();
                retVal = GetDRInfo(messageSend.DataBuffer, messageSend.Length, ref drPolicyData, ref lastError);
                if (!retVal)
                {
                    EventManager.WriteMessage(258, "GetDRInfo", EventLevel.Error, "GetDRInfo failed because of error:" + lastError);
                }
                else
                {
                    string ivStr = Utils.ByteArrayToHexStr(drPolicyData.IV);
                    uint accessFlag = 0;
                    retVal = IsFileAccessAuthorized(fileName,
                                                    userName,
                                                    processName,
                                                    ivStr,
                                                    ref encryptKey,
                                                    ref accessFlag,
                                                    ref lastError);

                    if (retVal && !string.IsNullOrEmpty(encryptKey))
                    {
                        byte[] keyArray = Utils.ConvertHexStrToByteArray(encryptKey);

                        //write the iv and key to the reply data buffer with format FilterAPI.AESDataBuffer
                        AESDataBuffer aesData = new AESDataBuffer();
                        if (messageSend.MessageType == (uint)FilterAPI.FilterCommand.FILTER_REQUEST_ENCRYPTION_IV_AND_KEY_AND_ACCESSFLAG)
                        {
                            aesData.AccessFlags = FilterAPI.ALLOW_MAX_RIGHT_ACCESS;
                        }
                        else
                        {
                            aesData.AccessFlags = 0;
                        }
                        aesData.IV = drPolicyData.IV;
                        aesData.IVLength = (uint)aesData.IV.Length;
                        aesData.EncryptionKey = keyArray;
                        aesData.EncryptionKeyLength = (uint)keyArray.Length;

                        byte[] aesDataArray = DigitalRightControl.ConvertAESDataToByteArray(aesData);
                        messageReply.DataBufferLength = (uint)aesDataArray.Length;
                        Array.Copy(aesDataArray, messageReply.DataBuffer, aesDataArray.Length);

                        messageReply.ReturnStatus = (uint)FilterAPI.NTSTATUS.STATUS_SUCCESS;

                    }
                    else
                    {
                        retVal = false;                        
                    }
                }
            }
            catch (Exception ex)
            {
                lastError = "GetFileAccessPermission exception." + ex.Message;
                EventManager.WriteMessage(340, "GetFileAccessPermission", EventLevel.Error,lastError);
                retVal = false;
            }

            if (!retVal)
            {
                byte[] errorBuffer = UnicodeEncoding.Unicode.GetBytes(lastError);
                Array.Copy(errorBuffer, messageSend.DataBuffer, errorBuffer.Length);

                messageSend.DataBufferLength = (uint)errorBuffer.Length;
            }
          

            return retVal;

        }

        static public bool AddDRInfoToFile(string fileName,
                                    string authorizedProcessNames,
                                    string unauthorizedProcessNames,
                                    string authorizedUserNames,
                                    string unauthorizedUserNames,
                                    DateTime expireTime,
                                    byte[] encryptIV,
                                    byte[] encryptKey,
                                    string accessFlags)
        {

            try
            {
                if (!Directory.Exists(GlobalConfig.DRInfoFolder))
                {
                    Directory.CreateDirectory(GlobalConfig.DRInfoFolder);
                }             

                string iv = Utils.ByteArrayToHexStr(encryptIV);
                string key = Utils.ByteArrayToHexStr(encryptKey);

                string drFilePath = GlobalConfig.DRInfoFolder + "\\" + iv + ".xml";
                Dictionary<string, string> keyValues = new Dictionary<string, string>();

                keyValues.Add("fileName", fileName);
                keyValues.Add("key", key);
                keyValues.Add("iv", iv);
                keyValues.Add("accessFlags", accessFlags);
                keyValues.Add("authorizedProcessNames", authorizedProcessNames.ToLower());
                keyValues.Add("unauthorizedProcessNames", unauthorizedProcessNames.ToLower());
                keyValues.Add("authorizedUserNames", authorizedUserNames.ToLower());
                keyValues.Add("unauthorizedUserNames", unauthorizedUserNames.ToLower());
                keyValues.Add("expireTime", expireTime.ToFileTime().ToString());
                keyValues.Add("creationTime", DateTime.Now.ToFileTime().ToString());

                Utils.SaveAppSetting(drFilePath, keyValues);

                return true;
            }
            catch (Exception ex)
            {
                return false;
            }

        }

     
    }
}
